@@ -73,7 +73,7 @@ builtinEnvironment[_infinity] = Procedure(
   defaults    = Tuple(),
   ellipsis    = False,
   environment = builtinEnvironment,
-  body        = lambda eval_,env:None, # FIXME
+  body        = lambda eval_,env:float('inf'), # FIXME
 )
 
 builtinEnvironment[_empty] = Procedure(
